// thread.cpp
// Revision 25-apr-2006


#include "thread.h"

#include "error.h"

#include "trace.h"


#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS

// #include <pthread.h>


class blassic::Thread::Internal {
public:
	Internal ();
	~Internal ();
	void start (Thread & th);
	void lock ();
	void unlock ();
	void waitcond ();
	void setcond ();
private:
	pthread_cond_t ptc;
	pthread_mutex_t ptm;
	pthread_t ptt;
};


namespace {


void * thread_function (void * param)
{
	blassic::Thread * pth= static_cast <blassic::Thread *> (param);
	pth->threadproc ();
	return 0;
}


} // namespace


blassic::Thread::Internal::Internal ()
{
}

blassic::Thread::Internal::~Internal ()
{
}

void blassic::Thread::Internal::start (blassic::Thread & th)
{
	pthread_mutex_init (& ptm, 0);
	if (pthread_cond_init (& ptc, NULL) != 0)
		throw ErrFunctionCall;
	if (pthread_create (& ptt, NULL, thread_function, & th) != 0)
		throw ErrFunctionCall;
}

void blassic::Thread::Internal::lock ()
{
	pthread_mutex_lock (& ptm);
}

void blassic::Thread::Internal::unlock ()
{
	pthread_mutex_unlock (& ptm);
}

void blassic::Thread::Internal::waitcond ()
{
	pthread_cond_wait (& ptc, & ptm);
}


void blassic::Thread::Internal::setcond ()
{
	//pthread_cond_signal (& ptc);
	pthread_cond_broadcast (& ptc);
}


#endif


blassic::Thread::Thread ()
	#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS
	:
	pin (new Internal)
	#endif
{
}

blassic::Thread::~Thread ()
{
	#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS
	delete pin;
	#endif
}


void blassic::Thread::start ()
{
	TRF;

	#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS
	pin->start (* this);
	#endif
}

void blassic::Thread::lock ()
{
	#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS
	pin->lock ();
	#endif
}

void blassic::Thread::unlock ()
{
	#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS
	pin->unlock ();
	#endif
}

void blassic::Thread::waitcond ()
{
	#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS
	pin->waitcond ();
	#endif
}

void blassic::Thread::setcond ()
{
	#if BLASSIC_HAS_THREADS && ! BLASSIC_CONFIG_NO_PTHREADS
	pin->setcond ();
	#endif
}


blassic::Locker::Locker (Thread & th_n) :
	th (th_n)
{
	th.lock ();
}

blassic::Locker::~Locker ()
{
	th.unlock ();
}


// End of thread.cpp
